//*********************************************************
//
//  UNIT:  F r a c t M a p             14.09.98,  03.09.99
//
//  create fractal map with dimension 2^nDim
//
//  the fractal map consist of a height and a color map.
//
//*********************************************************
unit FractMap;

interface

uses  Windows, SysUtils, Graphics,
      Graph, Palette;

const nDim   = 8;            {-> maxX=256}
      xShift = 16 - nDim;
      yShift = 16 - nDim;
      mapDim = 1 shl nDim;   { = 2^nDim}
      maxX   = mapDim - 1;
      maxY   = mapDim - 1;

const maxColor = 255;

type  tColorArray = array[0..maxColor] of tColor;

var   heightMap: tBitmap = nil;
      colorMap:  tBitmap = nil;

procedure CreateMap (var hm, cm: tBitmap);
procedure DeleteMap (var hm, cm: tBitmap);
procedure SelectMap (var hm, cm: tBitmap);

function  GetHeight (x,y: word): byte;
function  GetColor  (x,y: word): tColor;
procedure PutHeight (x,y: integer; height: byte);
procedure PutColor  (x,y: integer; color: tColor);
procedure SaveMap (filename: string);
procedure LoadMap (filename: string);
procedure DrawMap (x,y, ms: integer);
procedure ChangeColorMap;
procedure CorrectHeightMap;
procedure InitLandscapePalette;
procedure GenerateMap (fractalDim: integer);

var   cPal: tPalette;
      cMap: tColorArray;

{************************************************}
implementation

{------------------------------------------------}
procedure CreateMap (var hm, cm: tBitmap);
begin
  // height map
  if hm = nil
  then begin
    hm := tBitmap.Create;
    hm.PixelFormat := pf8Bit;
    hm.Width  := mapDim;
    hm.Height := mapDim;
    SetIntensityPalette (hm);
  end;

  // color map
  if cm = nil
  then begin
    cm := tBitmap.Create;
    cm.PixelFormat := pf8Bit;
    cm.Width  := mapDim;
    cm.Height := mapDim;
    InitLandscapePalette;
  end;

  SelectMap (hm, cm);
end;
{------------------------------------------------}
procedure DeleteMap (var hm, cm: tBitmap);
begin
  hm.Free;   hm := nil;
  cm.Free;   cm := nil;
end;
{------------------------------------------------}
procedure SelectMap (var hm, cm: tBitmap);
begin
  heightMap := hm;
  colorMap := cm;
end;

{================================================}
{ palette handling
{================================================}

const maxCG = 10;

type  irgb = record i,r,g,b:byte; end;
      tcol = array[0..maxCG]of irgb;   // color table

const minH = 85;   // minimum height = sea level

      ca: tcol
      =((i:   0;r:  0; g:  0; b:120)   {dunkelblau}
       ,(i:  50;r: 70; g: 90; b:255)   {blau}
       ,(i:  80;r:150; g:150; b:233)   {hellblau}
       ,(i:minH;r:235; g:225; b: 80)   {hellbeige}
       ,(i: 100;r:200; g:160; b: 70)   {dunkelbeige}
       ,(i: 120;r: 50; g:255; b:  0)   {hellgrn}
       ,(i: 180;r: 80; g:145; b:  0)   {dunkelgrn}
       ,(i: 185;r: 77; g: 77; b: 77)   {dunkelgrau}
       ,(i: 210;r:160; g:160; b:160)   {grau}
       ,(i: 225;r:215; g:215; b:215)   {hellgrau}
       ,(i: 255;r:255; g:255; b:255)); {weiss}

{------------------------------------------------}
procedure InitLandscapePalette;

var   ci,di, dc, first, last: word;
      cp1, cp2: irgb;

  function RGBValue (v1,v2: integer): byte;

  begin
    RGBValue := v1 + ((v2 - v1) * (di - first)) div dc;
  end;

begin
  for ci := 0 to maxCG-1
  do begin
    first := ca[ci].i;
    last := ca[ci+1].i;
    dc  := last - first;
    cp1 := ca[ci];
    cp2 := ca[ci+1];
    for di := first to last
    do with cPal[di]
    do begin
      rb := RGBValue (cp1.r, cp2.r);
      gb := RGBValue (cp1.g, cp2.g);
      bb := RGBValue (cp1.b, cp2.b);
    end;
  end;
  if sizeof (cPal) = sizeof(cMap)
  then move (cPal, cMap, sizeof (cMap));
end;

{================================================}
{ fractal map handling
{================================================}

{------------------------------------------------}
function GetHeight (x,y: word): byte;
var   linePtr: pByteArray;
begin
  linePtr := heightMap.ScanLine[y and maxY];
  GetHeight := linePtr^[x and maxX];
end;
{------------------------------------------------}
function GetColor (x,y: word): tColor;
var      linePtr: pByteArray;
begin
  linePtr := heightMap.ScanLine[y and maxY];
  GetColor := cMap[linePtr^[x and maxX]];
end;
{------------------------------------------------}
procedure PutHeight (x,y: integer; height: byte);
var   linePtr: pByteArray;
begin
  linePtr := heightMap.ScanLine[y and maxY];
  linePtr^[x and maxX] := height;
end;
{------------------------------------------------}
procedure PutColor (x,y: integer; color: tColor);
begin
end;

{------------------------------------------------}
{$I-,X+}

const heightMapFilename = 'Height.BMP';
      colorMapFilename  = 'Color.BMP';

{------------------------------------------------}
procedure SaveMap (filename: string);
begin
  SaveBitmap (heightMap, filename+heightMapFilename);
  SaveBitmap (colorMap, filename+colorMapFilename);
end;
{------------------------------------------------}
procedure LoadMap (filename: string);
begin
  LoadBitmap (heightMap, filename+heightMapFilename);
  LoadBitmap (colorMap, filename+colorMapFilename);
end;
{------------------------------------------------}
procedure DrawMap (x,y, ms: integer);
//var  xi,yi, xs,ys, xp,yp: word; lp: pByteArray;
begin
  CopyBitmap (colorMap, bgBitmap);
{
  xs := nDim - ms;
  ys := nDim - ms;
  for yi := 0 to (maxY shr ys)
  do begin
    yp := yi shl ys;
    lp := colorMap.bitmap.scanline[yp];
    for xi := 0 to (maxX shr xs)
    do begin
      xp := xi shl xs;
      PutPixel(x+xi, y+yi, lp^[xp]);
    end;
  end;
}
end;
{------------------------------------------------}
procedure ChangeColorMap;       //???
var   xi,yi: word;  rcol: tColor;
begin
  for yi := 0 to maxY
  do begin
    for xi := 0 to maxX
    do begin
      rcol := cMap[xi] - 8 + GetColor (xi+44,yi+55) div 16;
      if      rcol <  16 then rcol := 16
      else if rcol > 255 then rcol := 255;
      cmap[xi] := rcol;
    end;
  end;
end;
{------------------------------------------------}
procedure CopyHeight2ColorMap;
begin
  CopyBitmap (heightMap, colorMap);
end;
{------------------------------------------------}
{ set minimum height to sea level
{------------------------------------------------}
procedure CorrectHeightMap;
var   xi,yi: word;  lp: pByteArray;
begin
  for yi := 0 to maxY
  do begin
    lp := heightMap.Scanline[yi];
    for xi := 0 to maxX
    do if lp^[xi] < minH then lp^[xi] := minH;
  end;
end;
{------------------------------------------------}
{ create new random map
{ input:       fractDim: 1..100 %
{------------------------------------------------}
procedure RandomMap (fractDim: integer);

  procedure RandomHeight(x1,y1,x2,y2,max:word);
  var col: integer;
  begin
    col := (integer (GetHeight(x1,y1))
           +integer (GetHeight(x2,y2)))shr 1;
    col := col + random(max) - random(max);
    if col > 250 then col := 250
    else if col < 16 then col := 16;
    x1 := (x1+x2) shr 1;
    y1 := (y1+y2) shr 1;
    if (x1 <= maxX) and (y1 <= maxY)
    then PutHeight (x1,y1,col);
    if x1 = 0 then PutHeight (mapDim,y1,col);
    if y1 = 0 then PutHeight (x1,mapDim,col);
  end;

  procedure GenSub(x1,y1,x3,y3,max:word);
  var x2,y2:word;
  begin
    //sleep(5);   {only for testing}
    x2 := (x1+x3)shr 1;
    y2 := (y1+y3)shr 1;
    max := integer(max) * fractDim div 100;
    if x1 = 0 then RandomHeight (0,y1,0,y3,max); {links}
    if y1 = 0 then RandomHeight (x1,0,x3,0,max); {oben}
    RandomHeight (x1,y3,x3,y3,max); {unten}
    RandomHeight (x3,y1,x3,y3,max); {rechts}
    RandomHeight (x2,y1,x2,y3,max); {mitte}
    if x2-x1 > 1
    then begin
      GenSub (x1,y1,x2,y2,max);
      GenSub (x2,y1,x3,y2,max);
      GenSub (x1,y2,x2,y3,max);
      GenSub (x2,y2,x3,y3,max);
    end;
  end;

begin
  PutHeight (     0,     0,128+random(128));
  PutHeight (mapDim,mapDim,  GetHeight(0,0));
  PutHeight (     0,mapDim,  GetHeight(0,0));
  PutHeight (mapDim,     0,  GetHeight(0,0));
  GenSub (0,0, mapDim,mapDim, 255 * 100 div fractDim);
end;

{------------------------------------------------}
procedure GenerateMap (fractalDim: integer);
begin
  RandomMap (fractalDim);
  SetIntensityPalette (heightMap);
  SetAllPalette (cPal);
  SetBitmapPalette (colorMap, cPal);
  CopyHeight2ColorMap;
  ChangeColorMap;
  CorrectHeightMap;
end;

end.

