//*******************************************************************
//
//  UNIT:     P a l e t t e                 22.07.99,  31.08.99
//
//  palette handling for delphi 4.
//
//-------------------------------------------------------------------
//  InitPalette         initialize bitmap palette
//  SetAllPalette       set 256 color palette
//  GetAllPalette       return 256 color palette
//
//  SetPaletteColor     set one color value of color palette
//  PaletteColor        return color value of one palette color
//
//  SetBitmapPalette    set color palette of given Bitmap
//  CopyBitmapPalette   copy given bitmap color palette to active
//
//  SetIntensityPalette set gray scaled palette
//  CreateColorMap      create sine wave color palette
//
//  DrawRGBPalette      draw bar with palette colors
//
//*******************************************************************
unit palette;

interface

uses  Windows, SysUtils, Graphics,
      MathLib1, Graph;

type  tRGB = packed record   rb,gb,bb, fl: byte;   end;

      tPalette = array [0..255] of tRGB;

      tColorArray = array[0..9999] of tColor;
      pColorArray = ^tColorArray;

procedure InitPalette;
procedure SetAllPalette (var pal: tPalette);
procedure GetAllPalette (var pal: tPalette);

procedure SetPaletteColor (colorIndex: byte; color: tColor);
function PaletteColor (colorIndex: byte): tColor;

procedure SetBitmapPalette (bm: tBitmap; var pal: tPalette);
procedure GetBitmapPalette (bm: tBitmap; var pal: tPalette);
procedure CopyBitmapPalette (ph: HPALETTE);

procedure SetIntensityPalette (bm: tBitmap);
procedure CreateColorMap (rs,gs,bs: word);

procedure DrawPalette (var pal: tPalette; x,y,h: integer);
procedure DrawRGBPalette (x,y,h: integer);

var   activePalette: tMaxLogPalette;
      paletteHandle: HPALETTE;

//*********************************************************
implementation

const palColors = 256;
      palSize = palColors * sizeof (tRGB);

//---------------------------------------------------------
procedure ActivatePalette;
begin
  paletteHandle := Windows.CreatePalette(pLogPalette(@activePalette)^);
  if paletteHandle <> 0
  then bgBitmap.Palette := paletteHandle;
end;

//*********************************************************
procedure InitPalette;
var   ci: Integer;
begin
  activePalette.palVersion := $300;
  activePalette.palNumEntries := palColors;
  for ci := 16 to palColors-1
  do with activePalette.palPalEntry[ci]
  do begin
    peRed   := ci;
    peGreen := ci;
    peBlue  := ci;
  end;
  SetPaletteColor (Black,     clBlack);
  SetPaletteColor (Blue,      clBlue);
  SetPaletteColor (Green,     clGreen);
  SetPaletteColor (Cyan,      clCyan);
  SetPaletteColor (Red,       clRed);
  SetPaletteColor (Magenta,   clMagenta);
  SetPaletteColor (Brown,     clBrown);
  SetPaletteColor (LightGray, clLightGray);

  SetPaletteColor (DarkGray,     clDarkGray);
  SetPaletteColor (LightBlue,    clLightBlue);
  SetPaletteColor (LightGreen,   clLightGreen);
  SetPaletteColor (LightCyan,    clLightCyan);
  SetPaletteColor (LightRed,     clLightRed);
  SetPaletteColor (LightMagenta, clLightMagenta);
  SetPaletteColor (Yellow,       clYellow);
  SetPaletteColor (White,        clWhite);
end;

//*********************************************************
procedure SetAllPalette (var pal: tPalette);
begin
  Move (pal, activePalette.palPalEntry[0], palSize);
  ActivatePalette;
end;
//---------------------------------------------------------
procedure GetAllPalette (var pal: tPalette);
begin
  Move (activePalette.palPalEntry[0], pal, palSize);
end;

//*********************************************************
procedure SetPaletteColor(colorIndex: byte; color: tColor);
begin
  activePalette.palPalEntry[colorIndex] := tagPaletteEntry(color);
end;
//---------------------------------------------------------
function PaletteColor (colorIndex: byte): tColor;
begin
  result := longint(activePalette.palPalEntry[colorIndex]);
end;

//*********************************************************
// set given palette to given bitmap
//---------------------------------------------------------
procedure SetBitmapPalette (bm: tBitmap; var pal: tPalette);
var   bmPal: tMaxLogPalette;
      ph: HPALETTE;
begin
  bmPal := activePalette;
  Move (pal, bmPal.palPalEntry[0], palSize);
  ph := Windows.CreatePalette(pLogPalette(@bmPal)^);
  if ph <> 0
  then bm.Palette := ph;
end;
//*********************************************************
// copy palette colors of given bitmap to given palette
//---------------------------------------------------------
procedure GetBitmapPalette (bm: tBitmap; var pal: tPalette);
var   bmPal: tMaxLogPalette;
begin
  with bmPal
  do GetPaletteEntries(bm.Palette, 0, palNumEntries, palPalEntry);
  Move (bmPal.palPalEntry[0], pal, palSize);
end;
//*********************************************************
// set palette of a given bitmap as palette of bgBitmap
//---------------------------------------------------------
procedure CopyBitmapPalette (ph: HPALETTE);
begin
  if ph = 0 then exit;       // no palette ?
  // copy bitmap palette to activePalette
  with activePalette
  do GetPaletteEntries(ph, 0, palNumEntries, palPalEntry);
  // activate color palette
  ActivatePalette;
end;
//*********************************************************
//  set gray scaled palette
//---------------------------------------------------------
procedure SetIntensityPalette (bm: tBitmap);
var   bmPal: tMaxLogPalette;
      ph: HPALETTE;
      ci: integer;
begin
  bmPal := activePalette;
  for ci := 0 to 255
  do with bmPal
  do palPalEntry[ci] := tagPaletteEntry(RGB(ci,ci,ci));
  ph := Windows.CreatePalette(pLogPalette(@bmPal)^);
  if ph <> 0
  then bm.Palette := ph;
end;

{$Q-}
//*********************************************************
// create complete new color map
// input:      rs,gs,bs:  rgb steps
//---------------------------------------------------------
procedure CreateColorMap (rs,gs,bs: word);
var   ci, rVal, gVal, bVal: word;
begin
  randomize;
  rVal := random(65500);
  gVal := random(65500);
  bVal := random(65500);
  for ci := 1 to 255
  do begin
    with activePalette.palPalEntry[ci]
    do begin
      peRed   := sinB [rVal shr 8];
      peGreen := sinB [gVal shr 8];
      peBlue  := sinB [bVal shr 8];
    end;
    inc (rVal, rs);
    inc (gVal, gs);
    inc (bVal, bs);
  end;
  ActivatePalette;
end;

//*********************************************************
// call:       DrawPalette (pal, x,y,h);
// purpose:    draw given color palette at position x,y
// input:      pal: tPalette
//             x,y: integer;    left top position
//             h: integer;      height of color bar
//---------------------------------------------------------
procedure DrawPalette (var pal: tPalette;
                       x,y,h: integer);

var   w,xi,yi: word;   saveColor: tColor;

begin
  saveColor := GetColor;
  if y+h > GetMaxY then h := GetMaxY - y;
  SetColor (black);
  Rectangle (x, y, x+257, y+h+2);
  inc (x);
  inc (y);
  w := 255;
  if x+w >= pageX then w := pageX - x;
  // draw one palette color line
  if bgBitmap.PixelFormat > pf8Bit
  then for xi := 0 to w-1
  do PutPixel (x+xi, y, longint(pal[xi]))
  else for xi := 0 to w-1
  do PutPixel (x+xi, y, xi);
  // copy color lines
  for yi := y+1 to y+h-1
  do CopyLine (x,y,w, x,yi);
  SetColor (saveColor);
end;
//*********************************************************
// call:       DrawRGBPalette (x,y,h);
// purpose:    draw active color palette at position x,y
// input:      x,y: integer;    left top position
//             h: integer;      height of color bar
//---------------------------------------------------------
procedure DrawRGBPalette (x,y,h: integer);
begin
  DrawPalette (tPalette(activePalette.palPalEntry), x,y,h);
end;

//*********************************************************
begin
  InitPalette;
end.

